 /***********************************************************************************************
 * SOFTWARE RSL400
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE
 *
 * Description:
 * Example application for receiving UDP packets from RSL400
 *
 **************************************************************************************************/

#include "stdafx.h"
#include "rsl400\stateimage.h"
#include "rsl400\udpstateimage.h"

void main() {

	WSADATA wsaData;
	SOCKET receiverSocket;
	sockaddr_in receiverAddr;
	int port = 9090;
	int  receiveBufferLen = 2048000;
	char *receiveBuffer = new char[receiveBufferLen];
	sockaddr_in senderAddr;
	int senderAddrSize = sizeof(senderAddr);

	//-----------------------------------------------
	// Initialize Winsock
	WSAStartup(MAKEWORD(2,2), &wsaData);

	//-----------------------------------------------
	// Create a receiver socket to receive datagrams
	receiverSocket = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);

	//-----------------------------------------------
	// Bind the socket to any address and the specified port.
	receiverAddr.sin_family = AF_INET;
	receiverAddr.sin_port = htons(port);
	receiverAddr.sin_addr.s_addr = htonl(INADDR_ANY);

	bind(receiverSocket, (SOCKADDR *) &receiverAddr, sizeof(receiverAddr));

	//-----------------------------------------------
	// Call the recvfrom function to receive datagrams
	// on the bound socket.
	printf("Receiving datagrams...\n");
	
	// receive 100 packets
	for( int n = 0; n < 100; n++ ) {
		
		int received = recvfrom(receiverSocket, 
			receiveBuffer, 
			receiveBufferLen, 
			0, 
			(SOCKADDR *)&senderAddr, 
			&senderAddrSize);
		
		//-----------------------------------------------
		// abort in case of an error
		if ( received == SOCKET_ERROR ) {
			printf( "WSAGetLastError: %d\r\n", WSAGetLastError() );
			break;
		}

		RSL400::PUdpTelegramType udpTelegramType = (RSL400::PUdpTelegramType)receiveBuffer;

		printf( "--> Telegram Type: %d\r\n", udpTelegramType->Id);

		if ( udpTelegramType->Id == 1 ) {
			int test = sizeof( RSL400::UdpExtStateImageType1);
			RSL400::PUdpExtStateImageType1 udpExtStateImageType1 = (RSL400::PUdpExtStateImageType1)receiveBuffer;

			// output some data
			printf( "ScanNo : %d\r\n", udpExtStateImageType1->StateImage1.ScanNo );
			printf( "IsOssdA: %d\r\n", udpExtStateImageType1->StateImage1.IsOssdA );
			printf( "IsEA1  : %d\r\n", udpExtStateImageType1->StateImage1.IsEA1);
			printf( "IsEA2  : %d\r\n", udpExtStateImageType1->StateImage1.IsEA2);
			printf( "IsEA3  : %d\r\n", udpExtStateImageType1->StateImage1.IsEA3);
			printf( "IsEA4  : %d\r\n", udpExtStateImageType1->StateImage1.IsEA4);
		}
		else if ( udpTelegramType->Id == 6 ) {
			RSL400::PUdpBeamPacket t3 = (RSL400::PUdpBeamPacket)receiveBuffer;

			// output some data
			printf( "BlockNo: %d\r\n", t3->BlockNo);
		}
	}

	//-----------------------------------------------
	// Close the socket when finished receiving datagrams
	printf("Finished receiving. Closing socket.\n");
	closesocket(receiverSocket);

	//-----------------------------------------------
	// Clean up and exit.
	printf("Exiting.\n");
	WSACleanup();
	delete[] receiveBuffer;
	return;
}

